package tests;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import ncsa.hdf.hdf5lib.H5;
import ncsa.hdf.hdf5lib.HDF5Constants;
import ncsa.hdf.object.Attribute;
import ncsa.hdf.object.Dataset;
import ncsa.hdf.object.Datatype;
import ncsa.hdf.object.FileFormat;
import ncsa.hdf.object.Group;
import ncsa.hdf.object.h4.H4File;
import ncsa.hdf.object.h5.H5File;

public class Hdf5Tests {

	public static void main(String[] args) {
		
		if (args[0].equals("open")){
			File file = new File(args[1]);
			open_file(file);
			return;
			}
		
		if (args[0].equals("open4")){
			File file = new File(args[1]);
			open4_file(file);
			return;
			}
		
		if (args[0].equals("create")){
			File file = new File(args[1]);
			create_file(file);
			return;
			}
		
	}
	
	static void open_file(File file)  { 
		
		String fname = file.getAbsolutePath();
		FileFormat hdf_file = null;
		
		try{
			hdf_file = new H5File(fname);
			hdf_file.open();
			System.out.println("HDF5 format detected.");
		}catch (Exception ex){
			try{
				hdf_file.close();
				// Try HDF4
				hdf_file = new H4File(fname);
				hdf_file.open();
			}catch(Exception ex2){
				ex.printStackTrace();
				return;
				}
			System.out.println("HDF4 format detected.");
			}
		try{
			hdf_file.open();
			
			Group root = (Group)(( DefaultMutableTreeNode)hdf_file.getRootNode()).getUserObject();
			
			// List node and children recursively
			print_tree("", root);
			
			hdf_file.close();
		
		}catch (Exception ex){
			ex.printStackTrace();
			}
		
	}
	
	static void open4_file(File file)  { 
		
		String fname = file.getAbsolutePath();
		
		try{
			H4File h4file = new H4File(fname);
			h4file.open();
			
			Group root = (Group)(( DefaultMutableTreeNode)h4file.getRootNode()).getUserObject();
			
			// List node and children recursively
			print_tree("", root);
			
			h4file.close();
		
		}catch (Exception ex){
			ex.printStackTrace();
			}
		
	}
	
	private static void print_tree(String tab, Group node) throws Exception{
		
		System.out.println(tab + "Group: " + node.getName());
		// Attributes
		List<?> metadata = node.getMetadata();
		for (int i = 0; i < metadata.size(); i++){
			System.out.println(tab + " -> " + metadata.get(i).toString());
			}
		
		List<?> children = node.getMemberList();
		tab = tab + "\t";
		for (int i = 0; i < children.size(); i++){
			if (children.get(i) instanceof Group){
				print_tree(tab, (Group)children.get(i));
			}else{
				Dataset ds = ((Dataset)children.get(i));
				ds.getData();
				long dims[] = ds.getDims();
				String dimstr = " (";
				for (int j = 0; j < dims.length; j++){
					if (j > 0) dimstr = dimstr + ", ";
					dimstr = dimstr + dims[j];
					}
				dimstr = dimstr + ")";
				System.out.println(tab + "Dataset [" + ds.getDatatype().getDatatypeDescription() + "]: " + ds.getName() + dimstr);
				List<?> attributes = ds.getMetadata();
				for (int j = 0; j < attributes.size(); j++){
					Attribute attr = (Attribute)attributes.get(j);
					String value = string_for_value(attr.getValue());
					System.out.println(tab + " -> " + attr.getName() + " = " + value.toString());
					}
				}
			}
		
	}
	
	private static String string_for_value(Object obj){
		if (obj instanceof String[]){
			String[] array = (String[])obj;
			int dim = array.length;
			String value = "[" + array[0];
			for (int i = 1; i < dim; i++){
				value = value + ", " + array[i];
				}
			value = value + "]";
			return value;
			}
	
		if (obj instanceof int[]){
			int[] array = (int[])obj;
			int dim = array.length;
			String value = "[" + array[0];
			for (int i = 1; i < dim; i++){
				value = value + ", " + array[i];
				}
			value = value + "]";
			return value;
			}
	
		if (obj instanceof float[]){
			float[] array = (float[])obj;
			int dim = array.length;
			String value = "[" + array[0];
			for (int i = 1; i < dim; i++){
				value = value + ", " + array[i];
				}
			value = value + "]";
			return value;
			}
		
		if (obj instanceof double[]){
			double[] array = (double[])obj;
			int dim = array.length;
			String value = "[" + array[0];
			for (int i = 1; i < dim; i++){
				value = value + ", " + array[i];
				}
			value = value + "]";
			return value;
			}
	
		return obj.toString();
	}
	
	private static String string_for_type(Datatype type){
		
		
		switch(type.getDatatypeClass()){
		
			case Datatype.CLASS_FLOAT:
				return "32-bit Float";
				
			case Datatype.CLASS_INTEGER:
				return "32-bit Integer";
				
			case Datatype.CLASS_STRING:
				return "String";
				
			default:
				return type.getName();
		
			}
		
	}
	
	static void create_file(File file)  { 
		 int fid=-1, sid=-1, did=-1; 
		 long[] dims = { 4, 6 }; 
		 
		 try{
			 // Create a new file using default properties. 
			 fid = H5.H5Fcreate(file.getAbsolutePath(), HDF5Constants.H5F_ACC_TRUNC, 
			 HDF5Constants.H5P_DEFAULT, HDF5Constants.H5P_DEFAULT); 
			 
			 // Create the data space for the dataset. 
			 sid = H5.H5Screate_simple(2, dims, null); 
			 
			 // Create the dataset. 
			 if ((fid >= 0) && (sid >= 0)) 
			 did = H5.H5Dcreate(fid, "/dset", HDF5Constants.H5T_STD_I32BE, sid, 
			 HDF5Constants.H5P_DEFAULT); 
			 
			 // End access to the dataset and release resources used by it. 
			 if (did >= 0) 
			 H5.H5Dclose(did); 
			 
			 // Terminate access to the data space. 
			 if (sid >= 0) 
			 H5.H5Sclose(sid); 
			 
			 // Close the file. 
			 if (fid >= 0) 
			 H5.H5Fclose(fid); 
			 
			 System.out.println("Created HDF5 dataset file: '" + file.getAbsolutePath() + "'.");
		 
		 }catch (Exception ex){
			ex.printStackTrace(); 
		 	}
	} 

	

}
