package tests;

import java.awt.Dimension;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import mgui.datasources.DataTypes;
import mgui.geometry.Box3D;
import mgui.interfaces.stats.HistogramPlot;
import mgui.io.foreign.minc.MincVolumeMetadata;
import mgui.numbers.MguiNumber;
import mgui.stats.Histogram;

public class MincTests {

	public static void main(String[] args) {
		
		if (args[0].equals("load_minc_header")){
			load_minc_header(args[1]);
			}
		
		if (args[0].equals("load_minc1_header")){
			load_minc1_header(args[1]);
			}
		
		if (args[0].equals("load_minc_data")){
			load_minc_data(args[1]);
			}
		
	}
	
	
	private static void load_minc_header(String fn){
		
		MincVolumeMetadata metadata = new MincVolumeMetadata(new File(fn));
		System.out.println("Header loaded successfully: " + metadata.isInit());
		System.out.println("Dimension data loaded successfully: " + metadata.found_dimensions);
		System.out.println("Images data loaded successfully: " + metadata.found_images);
		
		int dims[] = metadata.getDataDims();
		System.out.println("\nVoxel dims: [" + dims[0] + "," + dims[1] + "," + dims[2] + "]");
		float[] geom_dims = metadata.getGeomDims();
		System.out.println("Geom dims: [" + geom_dims[0] + "," + geom_dims[1] + "," + geom_dims[2] + "]");
		Vector3f[] axes = metadata.getAxes();
		System.out.println("X-axis: [" + axes[0].getX() + "," + axes[0].getY() + "," + axes[0].getZ() + "]");
		System.out.println("Y-axis: [" + axes[1].getX() + "," + axes[1].getY() + "," + axes[1].getZ() + "]");
		System.out.println("Z-axis: [" + axes[2].getX() + "," + axes[2].getY() + "," + axes[2].getZ() + "]");
		Point3f origin = metadata.getOrigin();
		System.out.println("Origin: [" + origin.getX() + "," + origin.getY() + "," + origin.getZ() + "]");
		Box3D bounds = metadata.getBounds();
		System.out.println("Bounds:\n" + bounds.toString());
		
	}
	
	private static void load_minc1_header(String fn){
		
		MincVolumeMetadata metadata = new MincVolumeMetadata(new File(fn));
		System.out.println("Header loaded successfully: " + metadata.isInit());
		System.out.println("Dimension data loaded successfully: " + metadata.found_dimensions);
		System.out.println("Images data loaded successfully: " + metadata.found_images);
		
		int dims[] = metadata.getDataDims();
		System.out.println("\nVoxel dims: [" + dims[0] + "," + dims[1] + "," + dims[2] + "]");
		float[] geom_dims = metadata.getGeomDims();
		System.out.println("Geom dims: [" + geom_dims[0] + "," + geom_dims[1] + "," + geom_dims[2] + "]");
		Vector3f[] axes = metadata.getAxes();
		System.out.println("X-axis: [" + axes[0].getX() + "," + axes[0].getY() + "," + axes[0].getZ() + "]");
		System.out.println("Y-axis: [" + axes[1].getX() + "," + axes[1].getY() + "," + axes[1].getZ() + "]");
		System.out.println("Z-axis: [" + axes[2].getX() + "," + axes[2].getY() + "," + axes[2].getZ() + "]");
		Point3f origin = metadata.getOrigin();
		System.out.println("Origin: [" + origin.getX() + "," + origin.getY() + "," + origin.getZ() + "]");
		Box3D bounds = metadata.getBounds();
		System.out.println("Bounds:\n" + bounds.toString());
		
	}
	
	private static void load_minc_data(String fn){
		
		try{
			MincVolumeMetadata metadata = new MincVolumeMetadata(new File(fn));
			ArrayList<MguiNumber> values = metadata.readVolume(0, DataTypes.getType(DataTypes.FLOAT), null);
			System.out.println("Loaded MINC data.");
			
			Histogram hist = new Histogram(values, 80, 0, 500);
			HistogramPlot panel = new HistogramPlot(hist);
			
			JFrame frame = new JFrame("Histogram: " + fn);
			frame.setContentPane(panel);
			frame.setSize(400, 300);
			frame.getContentPane().setSize(400, 300);
			frame.getContentPane().setPreferredSize(new Dimension(500, 600));
			frame.pack();
			frame.setVisible(true);
			
			
			
		}catch (Exception ex){
			ex.printStackTrace();
			}
		
	}

}
